###
###  VE OS metrics monitoring module
###  Copyright (c) 2018 NEC Corporation
###

import subprocess
import traceback, sys
import os.path
import glob
import copy
import threading
import string
import os
import re 
import time 
import ve_common
import logging
import logging.handlers
from logging import Formatter

descriptors = []
veos_logger = None
_Worker_Thread = None
_Lock = threading.Lock() # synchronization lock

# Metric definition
OSMETRICS = [
    { "name": 'os_state', "time_max": 60,  "value_type": 'string', "units": '', 
      "slope": 'zero',   "format" : "%s", "description": 'State of VEOS' },
    { "name": 'vm_memory_size_free', "time_max": 60, "value_type": 'uint', "units": 'KB', 
      "slope": 'both',   "format" : "%d", "description": 'free memory size of VE'  },
    { "name": 'vm_memory_size_pfree', "time_max": 60,  "value_type": 'float', "units": '%', 
      "slope": 'both',   "format" : "%.2f", "description": 'percent of free memory size'  },
    { "name": 'load1',  "time_max": 60,  "value_type": 'float', "units": '', 
      "slope": 'both',   "format" : "%.2f", "description": 'load average for 1 minute'  },
    { "name": 'load5',  "time_max": 60,  "value_type": 'float', "units": '', 
      "slope": 'both',   "format" : "%.2f", "description": 'load average for 5 minutes'  },
    { "name": 'load15', "time_max": 60,  "value_type": 'float', "units": '', 
      "slope": 'both',   "format" : "%.2f", "description": 'load average for 15 minutes'  },
    { "name": 'vecpu_sys', "time_max": 60,  "value_type": 'float', "units": '%', 
      "slope": 'both',   "format" : "%.2f", "description": 'system time'  },
    { "name": 'vecpu_usr', "time_max": 60,  "value_type": 'float', "units": '%', 
      "slope": 'both',   "format" : "%.2f", "description": 'user time'  },
    { "name": 'vecpu_idle',"time_max": 60,  "value_type": 'float', "units": '%', 
      "slope": 'both',   "format" : "%.2f", "description": 'idle time'  },
    ]

DEFAULT_REFRESH = 60
NAME_PREFIX = 'veos_' 
VMSTAT_CMD = "/opt/nec/ve/bin/vmstat"
UPTIME_CMD = "/opt/nec/ve/bin/uptime"
MPSTAT_CMD = "/opt/nec/ve/bin/mpstat"

#logging message
FILE_OPEN_FAILED     = " Can not open "
FILE_READ_FAILED     = " Can not read "
FILE_CLOSE_FAILED    = " Can not close "
THREAD_ERROR_JOIN    = " Can not shutdown thread."
THREAD_ERROR_RELEASE = " Can not release thread."
THREAD_ERROR_START   = " Can not start thread."

class UpdateMetricThread(threading.Thread):
    
    def __init__(self, params, selfmetric, ve_list, sysfsd):
        threading.Thread.__init__(self)
        self.running      = False
        self.shuttingdown = False
        self.refresh_rate = DEFAULT_REFRESH
        if "RefreshRate" in params:
            self.refresh_rate = int(params["RefreshRate"])
        self.metric       = selfmetric
        self.ve_list      = ve_list
        self.totmem       = {}
        self.sysfs_directory = sysfsd

    def shutdown(self):
        self.shuttingdown = True
        if not self.running:
            return
        try:
            self.join()
        except Exception as e:
            output_logmsg(e, THREAD_ERROR_JOIN)
            os._exit(1)

    def run(self):
        self.running = True
        while not self.shuttingdown:
            self.update_metric()
            time.sleep(self.refresh_rate)

        self.running = False

    def update_metric(self):
        for ve_n in self.ve_list:
            sysfs_dir = self.sysfs_directory['ve_'+str(ve_n)+'sysfsd']
            
            #VE state is ONLINE
            if self.is_ve_state_online(sysfs_dir, ve_n) == True:
                self.get_value_os_state(ve_n,sysfs_dir)
                self.get_value_memory(ve_n)
                self.get_value_load(ve_n)
                self.get_value_system(ve_n)
                
            #VE state is OFFLINE
            else:
                self.get_value_os_state(ve_n,sysfs_dir)
                make_element(self.metric, "vm_memory_size_total", ve_n, 0)
                make_element(self.metric, "vm_memory_size_free",  ve_n, 0)
                make_element(self.metric, "vm_memory_size_pfree", ve_n, 0)
                make_element(self.metric, "load1",  ve_n, 0)
                make_element(self.metric, "load5",  ve_n, 0)
                make_element(self.metric, "load15", ve_n, 0)
                make_element(self.metric, "vecpu_usr",  ve_n, 0)
                make_element(self.metric, "vecpu_sys",  ve_n, 0)
                make_element(self.metric, "vecpu_idle", ve_n, 0)

    def is_ve_state_online(self, sysfs, ve_n):
        file_name = "ve_state"
        isOnline = False
        
        try:
           file = open(sysfs + "/" + file_name, "r")
        except Exception as e:
            output_logmsg(e, FILE_OPEN_FAILED + "ve_state" + " metric sensor file : " + sysfs + "/" + file_name)
            return False
        else:
            try:
                line = file.readline()
            except Exception as e:
                output_logmsg(e, FILE_READ_FAILED + "ve_state" + " metric sensor file : " + sysfs + "/" + file_name)
                return False
            else:
	        val = re.split("\s+", line)
	        _Lock.acquire()
	        
	        #VE state is ONLINE
	        if val[0] == '1':
	            isOnline = True
	            
	        try: 
	            _Lock.release()
	        except Exception as e:
	            output_logmsg(e, THREAD_ERROR_RELEASE)
	            return False
	        finally:
	            try:
	                file.close()
	            except Exception as e:
	                output_logmsg(e, FILE_CLOSE_FAILED + "ve_state" + " metric sensor file : " + sysfs + "/" + file_name)
	                return False
	            else:
	                return isOnline

    def get_value_os_state(self,ve_n,sysfs_dir):
        # OS state
        try:
            file = open(sysfs_dir + "/" + "os_state", 'r')
        except Exception as e:
            output_logmsg(e, FILE_OPEN_FAILED + sysfs_dir + "/os_state.")
            make_element(self.metric, "os_state", ve_n, 'UNKNOWN')
            return
        else:
            try:
                line = file.readline()
            except Exception as e:
                output_logmsg(e, FILE_READ_FAILED + sysfs_dir + "/os_state.")
                make_element(self.metric, "os_state", ve_n, 'UNKNOWN')
                return
            else:
	            _Lock.acquire()
	            val = re.split("\s+", line)
	            if val[0] == '0':
	                vstr = 'ONLINE'
	            elif val[0] == '1':
	                vstr = 'OFFLINE'
	            elif val[0] == '2':
	                vstr = 'INITIALIZING'
	            elif val[0] == '3':
	                vstr = 'TERMINATING'
	            else:
	                vstr = 'UNKNOWN'
	                
	            make_element(self.metric, "os_state", ve_n, vstr)
	            
	            try: 
	                _Lock.release()
	            except Exception as e:
	                output_logmsg(e, THREAD_ERROR_RELEASE)
	                make_element(self.metric, "os_state", ve_n, 'UNKNOWN')
	                return
	            finally:
	                try:
	                    file.close()
	                except Exception as e:
	                    output_logmsg(e, FILE_CLOSE_FAILED + sysfs_dir + "/os_state.")
	                    make_element(self.metric, "os_state", ve_n, 'UNKNOWN')
	                    return
	                else:
	                    #If VE state is OFFLINE, not collect Total Memory info.
	                    if self.is_ve_state_online(sysfs_dir, ve_n) != True:
	                        return
	    
        # Total memory
        try:
            file = open(sysfs_dir + "/" + "memory_size", 'r')
        except Exception as e:
            output_logmsg(e, FILE_OPEN_FAILED + sysfs_dir + "/memory_size failed.")
            make_element(self.metric, "vm_memory_size_total", ve_n, 0)
            return
        else:
            try:
                line = file.readline()
            except Exception as e:
                output_logmsg(e, FILE_READ_FAILED + sysfs_dir + "/memory_size failed.")
                make_element(self.metric, "vm_memory_size_total", ve_n, 0)
                return
            else:
                _Lock.acquire()
                val = re.split("\s+", line)
                make_element(self.metric, "vm_memory_size_total", ve_n, int(val[0]) * 1024 * 1024)
                self.totmem[ve_n] = float(val[0]) * 1024 * 1024
                try: 
                    _Lock.release()
                except Exception as e:
                    output_logmsg(e, THREAD_ERROR_RELEASE)
                    make_element(self.metric, "vm_memory_size_total", ve_n, 0)
                    return
                finally:
	                try:
	                    file.close()
	                except Exception as e:
	                    output_logmsg(e, FILE_CLOSE_FAILED + sysfs_dir + "/memory_size failed.")
	                    make_element(self.metric, "vm_memory_size_total", ve_n, 0)
	                    return

    def get_value_memory(self,ve_n):
        cmd="env VE_NODE_NUMBER=" + str(ve_n) + " " + VMSTAT_CMD
        lines = self.exec_cmd(cmd)
        
        make_element(self.metric, "vm_memory_size_free", ve_n, 0)
        make_element(self.metric, "vm_memory_size_pfree", ve_n,0)
        
        for v in re.split('(?:\r\n|\n)',lines):
            if len(v) == 0:
                break
            if self.totmem == {}:
            	break
            d = v.split()
            if d[0] == "procs":
                continue
            elif d[0] == "r":
                continue
            else:
                _Lock.acquire()                
                make_element(self.metric, "vm_memory_size_free", ve_n, int(d[3]))
                make_element(self.metric, "vm_memory_size_pfree", ve_n, (float(d[3]) / float(self.totmem[ve_n]) *100) )
                try: 
                    _Lock.release()
                except Exception as e:
                    make_element(self.metric, "vm_memory_size_free", ve_n, 0)
                    make_element(self.metric, "vm_memory_size_pfree", ve_n,0)
                    output_logmsg(e, THREAD_ERROR_RELEASE)
                    return

    def get_value_load(self,ve_n):
        cmd="env VE_NODE_NUMBER=" + str(ve_n) + " " + UPTIME_CMD
        lines = self.exec_cmd(cmd)
        
        if len(lines) == 0:
            make_element(self.metric, "load1", ve_n, 0)
            make_element(self.metric, "load5", ve_n, 0)
            make_element(self.metric, "load15", ve_n,0)
                    
        for v in re.split('(?:\r\n|\n)',lines):
            if len(v) == 0:
                break
            d = v.split('load average:')
            av = d[1].split(',')
            _Lock.acquire()
            make_element(self.metric, "load1", ve_n, float(av[0]))
            make_element(self.metric, "load5", ve_n, float(av[1]))
            make_element(self.metric, "load15", ve_n, float(av[2]))
            try:
                _Lock.release()
            except Exception as e:
                make_element(self.metric, "load1", ve_n, 0)
                make_element(self.metric, "load5", ve_n, 0)
                make_element(self.metric, "load15", ve_n,0)
                output_logmsg(e, THREAD_ERROR_RELEASE)
                return

    def get_value_system(self,ve_n):
        cmd="env LC_ALL=C VE_NODE_NUMBER=" + str(ve_n) + " " + MPSTAT_CMD
        lines = self.exec_cmd(cmd)
        
        if len(lines) == 0:
            make_element(self.metric, "vecpu_usr", ve_n, 0)
            make_element(self.metric, "vecpu_sys", ve_n, 0)
            make_element(self.metric, "vecpu_idle", ve_n,0)
        
        for v in re.split('(?:\r\n|\n)',lines):
            if len(v) == 0:
                continue
            d = v.split()
            _Lock.acquire()
            if d[1] == 'all':
                make_element(self.metric, "vecpu_usr", ve_n, float(d[2]))
                make_element(self.metric, "vecpu_sys", ve_n, float(d[4]))
                make_element(self.metric, "vecpu_idle", ve_n, float(d[11]))
            try:
                _Lock.release()
            except Exception as e:
                make_element(self.metric, "vecpu_usr", ve_n, 0)
                make_element(self.metric, "vecpu_sys", ve_n, 0)
                make_element(self.metric, "vecpu_idle", ve_n,0)
                output_logmsg(e, THREAD_ERROR_RELEASE)
                return

    def exec_cmd(self,command):
        proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        pout, perr = proc.communicate()
        ecode = proc.wait()
        if ecode != 0:
            veos_logger.warning("Can not execute " + command)
            return ""
        return(pout)

    def metric_of(self, name):
        val = 0
        if name in self.metric:
            _Lock.acquire()
            if self.metric[name] != None:
                val = self.metric[name]
                veos_logger.debug(name + " = " + str(self.metric[name]))
            try:
                _Lock.release()
            except Exception as e:
                output_logmsg(e, THREAD_ERROR_RELEASE)
        return val

def output_logmsg(e, msg):
    #if exception attribute has errno, output errno
    if hasattr(e, "errno") == True: 
        veos_logger.warning("errno=" + str(e.errno) + msg)
    else:
        veos_logger.warning(msg)

def get_sysfsdir(ve_list):
    sysfsd = {}
    SYSTEM_DIR = "/sys"

    for ve in ve_list:
        dir = ve_common.get_sysdir(ve).rstrip("\n")
        
        if dir == "":
            return {}
        elif dir[0] != '/':
            return {}
            
        sys_dir = SYSTEM_DIR + dir
        
        sysfsd['ve_'+str(ve)+'sysfsd'] = sys_dir

    return sysfsd

def make_element(selfmetric, name, ve_n, val):
    for desc in OSMETRICS:
        if desc['name'] == name:
            metric_name = NAME_PREFIX + str(ve_n) + "_" + desc['name']
            selfmetric[metric_name] = val
            break

def metric_of(name):
    return _Worker_Thread.metric_of(name)

def metric_init(params):
    global descriptors, _Worker_Thread, veos_logger

    veos_logger = logging.getLogger('VEOS_LOGGER')
    veos_logger.setLevel(logging.DEBUG)
    
    handler = logging.handlers.SysLogHandler(address = "/dev/log")
    formatter = Formatter(fmt='%(levelname)s - VEOS : %(message)s')
    handler.formatter = formatter
    veos_logger.addHandler(handler)

    ve_base_info = ve_common.get_ve_base_info()
    
    if 've_list' not in ve_base_info:
        veos_logger.warning("Getting the number of VE failed.")
        return {}
        
    sysfsd = get_sysfsdir(ve_base_info['ve_list'])
    
    if sysfsd == {}:
        veos_logger.warning("Executing udevadm command failed.")
        return {}

    selfmetric = {}
    for ve in ve_base_info['ve_list']:
        for metric in OSMETRICS:
            d = copy.copy(metric)
            d['name'] = NAME_PREFIX + str(ve) + '_' + d['name']
            d['call_back'] = metric_of
            d['groups'] = "ve" +  str(ve)
            descriptors.append(d)
            selfmetric[d['name']] = 0
            
    _Worker_Thread = UpdateMetricThread(params, selfmetric, ve_base_info['ve_list'], sysfsd)
    try:
        _Worker_Thread.start()
    except Exception as e:
        output_logmsg(e, THREAD_ERROR_START)
    finally:
        return descriptors 

def metric_cleanup():
    _Worker_Thread.shutdown()


#This code is for debugging and unit testing
if __name__ == '__main__':
    try:
        metric_init({})
        
        ve_base_info = ve_common.get_ve_base_info()
        
        if len(ve_base_info) == 0:
            os._exit(1)
        
        for ve in ve_base_info['ve_list']:
            while True:
                for d in descriptors:
                    v = d['call_back'](d['name'])
                    print ('value for %s is '+d['format']) % (d['name'],  v)
                time.sleep(5)
    except KeyboardInterrupt:
        time.sleep(0.2)
        os._exit(1)
    except:
        traceback.print_exc()
        os._exit(1)
